#!/usr/bin/python3
"""
Allow buildbot to run jobs dynamically defined a a project repo
Requires docker
"""

# TODO: change "ghcontext" in master.cfg to interpolate the job name
# TODO: write upload script
# TODO: write hook (perhaps the default hook is ok), authentication for hook
# TODO: write hook job, maybe also a dockerfile?
# TODO: what to do about non-docker jobs

from os import walk
from os.path import exists, isfile, join, listdir
import requests

ty = '/change_hook/base'
request_headers = {
    'Content-Type': 'application/x-www-form-urlencoded',
    'Accept': 'text/plain'
}
request_data = {
    'project': 'testproject',
    'comments': 'testcomment',
}


def list_jobs(directory=".buildbot"):
    """
    list jobs found in a directory
    """
    results = []
    for _ in next(walk(directory))[1]:
        if exists(join(directory, _, "Dockerfile")) \
            and (exists(join(directory, _, "build.sh"))
                 or exists(join(directory, _, "test.sh"))
                 ):
            results.append(_)

    return results


def find_artifacts(directory="out"):
    """
    find artifacts (any file) in a directory
    """
    for _ in listdir(directory):
        if not isfile(join(directory, _)):
            continue
        return join(directory, _)

def trigger_child_hooks(buildbotUrl:str, directory=".buildbot"):
    request_url = buildbotUrl + ty
    
    # List all jobs in the directory
    jobs = list_jobs(directory)

    # Check if build.sh or test.sh exists in each of the jobs
    for job in jobs:
        build_script_exists = False
        test_script_exists = False
        if exists(join(directory, job, "build.sh")):
            build_script_exists = True
        if exists(join(directory, job, "test.sh")):
            test_script_exists = True
        
        # make a post request
        request_data['dockerfile'] = open(join(directory, job, "Dockerfile"), "r").read()
        request_data['build_script_available'] = str(build_script_exists)
        request_data['test_script_available'] = str(test_script_exists)
        requests.post(request_url, headers=request_headers, data=request_data)

if __name__ == "__main__":
    # expect jobname, repository, branch from command line args
    import sys
    if len(sys.argv) == 5:
        jobname = sys.argv[1]
        repository = sys.argv[2]
        branch = sys.argv[3]
        buildbotUrl = sys.argv[4]

        # add these into the request_data
        request_data['jobname'] = jobname
        request_data['repository'] = repository
        request_data['branch'] = branch

        trigger_child_hooks(buildbotUrl)
    else:
        print("Usage: python3 multibuild.py <jobname> <repository> <branch> <buildbotUrl>")